@@ -1,4 +1,4 @@
-/* $Id: ios_dev.m 4909 2014-09-01 04:03:31Z riza $ */
+/* $Id: ios_dev.m 5054 2015-04-08 10:09:37Z ming $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -49,14 +49,30 @@ static ios_fmt_info ios_fmts[] =
     { PJMEDIA_FORMAT_I420, kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange }
 };
 
-/* qt device info */
+typedef struct ios_supported_size
+{
+    pj_size_t supported_size_w;
+    pj_size_t supported_size_h;
+    NSString *preset_str;
+} ios_supported_size;
+
+/* Set the preset_str on set_preset_str method. */
+static ios_supported_size ios_sizes[] =
+{
+    { 352, 288, NULL },
+    { 640, 480, NULL },
+    { 1280, 720, NULL },
+    { 1920, 1080, NULL }
+};
+
+/* ios device info */
 struct ios_dev_info
 {
     pjmedia_vid_dev_info	 info;
     AVCaptureDevice             *dev;
 };
 
-/* qt factory */
+/* ios factory */
 struct ios_factory
 {
     pjmedia_vid_dev_factory	 base;
@@ -90,12 +106,14 @@ struct ios_stream
     unsigned		    bytes_per_row;
     unsigned		    frame_size;         /**< Frame size (bytes)*/
     pj_bool_t               is_planar;
+    NSLock 		   *frame_lock;
+    void                   *capture_buf;
     
     AVCaptureSession		*cap_session;
     AVCaptureDeviceInput	*dev_input;
     AVCaptureVideoDataOutput	*video_output;
     VOutDelegate		*vout_delegate;
-    void                        *capture_buf;
+    dispatch_queue_t 		 queue;
     AVCaptureVideoPreviewLayer  *prev_layer;
     
     void		*render_buf;
@@ -105,10 +123,6 @@ struct ios_stream
     
     pj_timestamp	 frame_ts;
     unsigned		 ts_inc;
-
-    pj_bool_t		 thread_initialized;
-    pj_thread_desc	 thread_desc;
-    pj_thread_t		*thread;
 };
 
 
@@ -140,6 +154,8 @@ static pj_status_t ios_stream_set_cap(pjmedia_vid_dev_stream *strm,
 				      pjmedia_vid_dev_cap cap,
 				      const void *value);
 static pj_status_t ios_stream_start(pjmedia_vid_dev_stream *strm);
+static pj_status_t ios_stream_get_frame(pjmedia_vid_dev_stream *strm,
+                                        pjmedia_frame *frame);
 static pj_status_t ios_stream_put_frame(pjmedia_vid_dev_stream *strm,
 					const pjmedia_frame *frame);
 static pj_status_t ios_stream_stop(pjmedia_vid_dev_stream *strm);
@@ -163,12 +179,19 @@ static pjmedia_vid_dev_stream_op stream_op =
     &ios_stream_get_cap,
     &ios_stream_set_cap,
     &ios_stream_start,
-    NULL,
+    &ios_stream_get_frame,
     &ios_stream_put_frame,
     &ios_stream_stop,
     &ios_stream_destroy
 };
 
+static void set_preset_str()
+{
+    ios_sizes[0].preset_str = AVCaptureSessionPreset352x288;
+    ios_sizes[1].preset_str = AVCaptureSessionPreset640x480;
+    ios_sizes[2].preset_str = AVCaptureSessionPreset1280x720;
+    ios_sizes[3].preset_str = AVCaptureSessionPreset1920x1080;
+}
 
 /****************************************************************************
  * Factory operations
@@ -199,6 +222,8 @@ static pj_status_t ios_factory_init(pjmedia_vid_dev_factory *f)
     unsigned i, l, first_idx, front_idx = -1;
     enum { MAX_DEV_COUNT = 8 };
     
+    set_preset_str();
+    
     /* Initialize input and output devices here */
     qf->dev_info = (struct ios_dev_info*)
 		   pj_pool_calloc(qf->pool, MAX_DEV_COUNT,
@@ -235,7 +260,7 @@ static pj_status_t ios_factory_init(pjmedia_vid_dev_factory *f)
                             sizeof(qdi->info.name));
             pj_ansi_strncpy(qdi->info.driver, "iOS", sizeof(qdi->info.driver));
             qdi->info.dir = PJMEDIA_DIR_CAPTURE;
-            qdi->info.has_callback = PJ_TRUE;
+            qdi->info.has_callback = PJ_FALSE;
             qdi->info.caps = PJMEDIA_VID_DEV_CAP_INPUT_PREVIEW |
 		    	     PJMEDIA_VID_DEV_CAP_SWITCH;
             qdi->dev = device;
@@ -268,13 +293,35 @@ static pj_status_t ios_factory_init(pjmedia_vid_dev_factory *f)
             {
                 continue;
             }
+            
+            if (qdi->info.dir == PJMEDIA_DIR_RENDER) {
+                fmt = &qdi->info.fmt[qdi->info.fmt_cnt++];
+                pjmedia_format_init_video(fmt,
+                                          ios_fmts[l].pjmedia_format,
+                                          DEFAULT_WIDTH,
+                                          DEFAULT_HEIGHT,
+                                          DEFAULT_FPS, 1);
+            } else {
+                int m;
+                AVCaptureDevice *dev = qdi->dev;
                 
-	    fmt = &qdi->info.fmt[qdi->info.fmt_cnt++];
-	    pjmedia_format_init_video(fmt,
-				      ios_fmts[l].pjmedia_format,
-				      DEFAULT_WIDTH,
-				      DEFAULT_HEIGHT,
-				      DEFAULT_FPS, 1);	
+                /* Set supported size for capture device */
+                for(m = 0; m < PJ_ARRAY_SIZE(ios_sizes) &&
+                           qdi->info.fmt_cnt<PJMEDIA_VID_DEV_INFO_FMT_CNT;
+                    m++)
+                {
+                    if ([dev supportsAVCaptureSessionPreset:
+                                                       ios_sizes[m].preset_str])
+                    {
+                        fmt = &qdi->info.fmt[qdi->info.fmt_cnt++];
+                        pjmedia_format_init_video(fmt,
+                                                  ios_fmts[l].pjmedia_format,
+                                                  ios_sizes[m].supported_size_w,
+                                                  ios_sizes[m].supported_size_h,
+                                                  DEFAULT_FPS, 1);
+                    }
+                }                
+            }
 	}
     }
     
@@ -387,9 +434,15 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
 		      didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
 		      fromConnection:(AVCaptureConnection *)connection
 {
-    pjmedia_frame frame = {0};
     CVImageBufferRef img;
 
+    /* Refrain from calling pjlib functions which require thread registration
+     * here, since according to the doc, dispatch queue cannot guarantee
+     * the reliability of underlying functions required by PJSIP to perform
+     * the registration, such as pthread_self() and pthread_getspecific()/
+     * pthread_setspecific().
+     */
+
     if (!sampleBuffer)
 	return;
     
@@ -399,10 +452,8 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
     /* Lock the base address of the pixel buffer */
     CVPixelBufferLockBaseAddress(img, kCVPixelBufferLock_ReadOnly);
     
-    frame.type = PJMEDIA_FRAME_TYPE_VIDEO;
-    frame.size = stream->frame_size;
-    frame.timestamp.u64 = stream->frame_ts.u64;
-    
+
+    [stream->frame_lock lock];
     if (stream->is_planar && stream->capture_buf) {
         if (stream->param.fmt.id == PJMEDIA_FORMAT_I420) {
             /* kCVPixelFormatType_420YpCbCr8BiPlanar* is NV12 */
@@ -450,32 +501,39 @@ static pj_status_t ios_factory_default_param(pj_pool_t *pool,
                     p += (stride - stream->size.w);
                 }
             }
-
-            frame.buf = stream->capture_buf;
         }
     } else {
-        frame.buf = CVPixelBufferGetBaseAddress(img);
-    }
-    
-    if (stream->vid_cb.capture_cb) {
-        if (stream->thread_initialized == 0 || !pj_thread_is_registered())
-        {
-            pj_bzero(stream->thread_desc, sizeof(pj_thread_desc));
-            pj_thread_register("ios_vdev", stream->thread_desc,
-                               &stream->thread);
-            stream->thread_initialized = 1;
-        }
-
-        (*stream->vid_cb.capture_cb)(&stream->base, stream->user_data, &frame);
+        pj_memcpy(stream->capture_buf, CVPixelBufferGetBaseAddress(img),
+                  stream->frame_size);
     }
 
     stream->frame_ts.u64 += stream->ts_inc;
+    [stream->frame_lock unlock];
     
     /* Unlock the pixel buffer */
     CVPixelBufferUnlockBaseAddress(img, kCVPixelBufferLock_ReadOnly);
 }
 @end
 
+static pj_status_t ios_stream_get_frame(pjmedia_vid_dev_stream *strm,
+                                        pjmedia_frame *frame)
+{
+    struct ios_stream *stream = (struct ios_stream *)strm;
+
+    frame->type = PJMEDIA_FRAME_TYPE_VIDEO;
+    frame->bit_info = 0;
+    pj_assert(frame->size >= stream->frame_size);
+    frame->size = stream->frame_size;
+    frame->timestamp.u64 = stream->frame_ts.u64;
+    
+    [stream->frame_lock lock];
+    pj_memcpy(frame->buf, stream->capture_buf, stream->frame_size);
+    [stream->frame_lock unlock];
+    
+    return PJ_SUCCESS;
+}
+
+
 static ios_fmt_info* get_ios_format_info(pjmedia_format_id id)
 {
     unsigned i;
@@ -575,37 +633,29 @@ static pj_status_t ios_factory_create_stream(
     strm->is_planar = vfi->plane_cnt > 1;
 
     if (param->dir & PJMEDIA_DIR_CAPTURE) {
-        NSString *size_preset_str[] = {
-            AVCaptureSessionPreset352x288,
-            AVCaptureSessionPreset640x480,
-            AVCaptureSessionPreset1280x720,
-            AVCaptureSessionPreset1920x1080
-        };
-        pj_size_t supported_size_w[] = { 352, 640, 1280, 1920 };
-        pj_size_t supported_size_h[] = { 288, 480,  720, 1080 };
-        pj_size_t supported_size[] = { 352*288, 640*480, 1280*720, 1920*1080 };
-        pj_size_t requested_size = strm->size.w * strm->size.h;
         int i;
-        
+	        
         /* Create capture stream here */
 	strm->cap_session = [[AVCaptureSession alloc] init];
 	if (!strm->cap_session) {
+	    PJ_LOG(2, (THIS_FILE, "Unable to create AV capture session"));
 	    status = PJ_ENOMEM;
 	    goto on_error;
 	}
-        
         AVCaptureDevice *dev = qf->dev_info[param->cap_id].dev;
-        
-	/* Find the closest supported size */
-        for(i = PJ_ARRAY_SIZE(supported_size)-1; i > 0; --i) {
-            if (![dev supportsAVCaptureSessionPreset: size_preset_str[i]])
-                continue;
-            if (supported_size[i-1] < requested_size)
+ 
+        for (i = PJ_ARRAY_SIZE(ios_sizes)-1; i > 0; --i) {
+            if ((vfd->size.w == ios_sizes[i].supported_size_w) &&
+                (vfd->size.h == ios_sizes[i].supported_size_h))
+            {
                 break;
+            }
         }
-        strm->cap_session.sessionPreset = size_preset_str[i];
-        vfd->size.w = supported_size_w[i];
-        vfd->size.h = supported_size_h[i];
+        
+        strm->cap_session.sessionPreset = ios_sizes[i].preset_str;
+        
+        vfd->size.w = ios_sizes[i].supported_size_w;
+        vfd->size.h = ios_sizes[i].supported_size_h;
         strm->size = vfd->size;
         strm->bytes_per_row = strm->size.w * vfi->bpp / 8;
         strm->frame_size = strm->bytes_per_row * strm->size.h;
@@ -629,39 +679,52 @@ static pj_status_t ios_factory_create_stream(
 	strm->dev_input = [AVCaptureDeviceInput
 			   deviceInputWithDevice:dev
 			   error: &error];
-	if (!strm->dev_input) {
+	if (error || !strm->dev_input) {
+	    PJ_LOG(2, (THIS_FILE, "Unable to get input capture device"));
 	    status = PJMEDIA_EVID_SYSERR;
 	    goto on_error;
 	}
-	[strm->cap_session addInput:strm->dev_input];
 	
-	strm->video_output = [[[AVCaptureVideoDataOutput alloc] init]
-			      autorelease];
-	if (!strm->video_output) {
+	if ([strm->cap_session canAddInput:strm->dev_input]) {
+	    [strm->cap_session addInput:strm->dev_input];
+	} else {
+	    PJ_LOG(2, (THIS_FILE, "Unable to add input capture device"));
 	    status = PJMEDIA_EVID_SYSERR;
 	    goto on_error;
 	}
+	
+	strm->video_output = [[AVCaptureVideoDataOutput alloc] init];
+	if (!strm->video_output) {
+	    PJ_LOG(2, (THIS_FILE, "Unable to create AV video output"));
+	    status = PJ_ENOMEM;
+	    goto on_error;
+	}
         
-        strm->video_output.alwaysDiscardsLateVideoFrames = YES;
-	[strm->cap_session addOutput:strm->video_output];
-	
 	/* Configure the video output */
-	strm->vout_delegate = [VOutDelegate alloc];
-	strm->vout_delegate->stream = strm;
-	dispatch_queue_t queue = dispatch_queue_create("myQueue", NULL);
-	[strm->video_output setSampleBufferDelegate:strm->vout_delegate
-                                              queue:queue];
-	dispatch_release(queue);
-	
+        strm->video_output.alwaysDiscardsLateVideoFrames = YES;
 	strm->video_output.videoSettings =
 	    [NSDictionary dictionaryWithObjectsAndKeys:
 			  [NSNumber numberWithInt:ifi->ios_format],
 			  kCVPixelBufferPixelFormatTypeKey, nil];
+
+	strm->vout_delegate = [VOutDelegate alloc];
+	strm->vout_delegate->stream = strm;
+	strm->queue = dispatch_queue_create("vout_queue",
+					    DISPATCH_QUEUE_SERIAL);
+	[strm->video_output setSampleBufferDelegate:strm->vout_delegate
+                            queue:strm->queue];
         
-        /* Prepare capture buffer if it's planar format */
-        if (strm->is_planar)
-            strm->capture_buf = pj_pool_alloc(strm->pool, strm->frame_size);
-        
+        if ([strm->cap_session canAddOutput:strm->video_output]) {
+	    [strm->cap_session addOutput:strm->video_output];
+	} else {
+	    PJ_LOG(2, (THIS_FILE, "Unable to add video data output"));
+	    status = PJMEDIA_EVID_SYSERR;
+	    goto on_error;
+	}
+	
+	strm->capture_buf = pj_pool_alloc(strm->pool, strm->frame_size);
+	strm->frame_lock = [[NSLock alloc]init];
+	
         /* Native preview */
         if (param->flags & PJMEDIA_VID_DEV_CAP_INPUT_PREVIEW) {
             ios_stream_set_cap(&strm->base, PJMEDIA_VID_DEV_CAP_INPUT_PREVIEW,
@@ -780,7 +843,6 @@ static pj_status_t ios_stream_set_cap(pjmedia_vid_dev_stream *s,
             /* Create view, if none */
 	    if (!strm->render_view)
 	        ios_init_view(strm);
-	    
             
             /* Preview layer instantiation should be in main thread! */
             dispatch_async(dispatch_get_main_queue(), ^{
@@ -966,8 +1028,10 @@ static pj_status_t ios_stream_start(pjmedia_vid_dev_stream *strm)
             });
         }
     
-	if (![stream->cap_session isRunning])
+	if (![stream->cap_session isRunning]) {
+	    PJ_LOG(3, (THIS_FILE, "Unable to start iOS capture session"));
 	    return PJ_EUNKNOWN;
+	}
     }
     
     return PJ_SUCCESS;
@@ -979,7 +1043,7 @@ static pj_status_t ios_stream_put_frame(pjmedia_vid_dev_stream *strm,
 					const pjmedia_frame *frame)
 {
     struct ios_stream *stream = (struct ios_stream*)strm;
-    
+
     if (stream->frame_size >= frame->size)
         pj_memcpy(stream->render_buf, frame->buf, frame->size);
     else
@@ -998,18 +1062,17 @@ static pj_status_t ios_stream_stop(pjmedia_vid_dev_stream *strm)
 {
     struct ios_stream *stream = (struct ios_stream*)strm;
 
-    PJ_UNUSED_ARG(stream);
-
+    if (!stream->cap_session || ![stream->cap_session isRunning])
+        return PJ_SUCCESS;
+    
     PJ_LOG(4, (THIS_FILE, "Stopping iOS video stream"));
 
-    if (stream->cap_session && [stream->cap_session isRunning]) {
-        if ([NSThread isMainThread]) {
+    if ([NSThread isMainThread]) {
+	[stream->cap_session stopRunning];
+    } else {
+        dispatch_sync(dispatch_get_main_queue(), ^{
             [stream->cap_session stopRunning];
-        } else {
-            dispatch_sync(dispatch_get_main_queue(), ^{
-                [stream->cap_session stopRunning];
-            });
-        }
+        });
     }
     
     return PJ_SUCCESS;
@@ -1026,22 +1089,24 @@ static pj_status_t ios_stream_destroy(pjmedia_vid_dev_stream *strm)
     ios_stream_stop(strm);
     
     if (stream->cap_session) {
-        [stream->cap_session removeInput:stream->dev_input];
+        if (stream->dev_input) {
+            [stream->cap_session removeInput:stream->dev_input];
+            stream->dev_input = nil;
+        }
         [stream->cap_session removeOutput:stream->video_output];
 	[stream->cap_session release];
 	stream->cap_session = nil;
-    }    
-    if (stream->dev_input) {
-        stream->dev_input = nil;
     }
  
+    if (stream->video_output) {
+        [stream->video_output release];
+        stream->video_output = nil;
+    }
+
     if (stream->vout_delegate) {
 	[stream->vout_delegate release];
 	stream->vout_delegate = nil;
     }
-    if (stream->video_output) {
-        stream->video_output = nil;
-    }
 
     if (stream->prev_layer) {
         CALayer *prev_layer = stream->prev_layer;
@@ -1066,6 +1131,16 @@ static pj_status_t ios_stream_destroy(pjmedia_vid_dev_stream *strm)
         stream->render_data_provider = nil;
     }
 
+    if (stream->queue) {
+        dispatch_release(stream->queue);
+        stream->queue = nil;
+    }
+    
+    if (stream->frame_lock) {
+        [stream->frame_lock release];
+        stream->frame_lock = nil;
+    }
+
     pj_pool_release(stream->pool);
 
     return PJ_SUCCESS;
