@@ -1,4 +1,4 @@
-/* $Id: sip_transaction.c 4839 2014-05-08 09:21:04Z bennylp $ */
+/* $Id: sip_transaction.c 4992 2015-03-06 06:09:22Z ming $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -1007,17 +1007,20 @@ static pj_status_t tsx_create( pjsip_module *tsx_user,
     
     if (grp_lock) {
 	tsx->grp_lock = grp_lock;
+        
+        pj_grp_lock_add_ref(tsx->grp_lock);
+        pj_grp_lock_add_handler(tsx->grp_lock, tsx->pool, tsx, &tsx_on_destroy);
     } else {
-	status = pj_grp_lock_create(pool, NULL, &tsx->grp_lock);
+	status = pj_grp_lock_create_w_handler(pool, NULL, tsx, &tsx_on_destroy,
+					      &tsx->grp_lock);
 	if (status != PJ_SUCCESS) {
 	    pjsip_endpt_release_pool(mod_tsx_layer.endpt, pool);
 	    return status;
 	}
+	
+	pj_grp_lock_add_ref(tsx->grp_lock);
     }
 
-    pj_grp_lock_add_ref(tsx->grp_lock);
-    pj_grp_lock_add_handler(tsx->grp_lock, tsx->pool, tsx, &tsx_on_destroy);
-
     status = pj_mutex_create_simple(pool, tsx->obj_name, &tsx->mutex_b);
     if (status != PJ_SUCCESS) {
 	tsx_shutdown(tsx);
@@ -1308,8 +1311,12 @@ PJ_DEF(pj_status_t) pjsip_tsx_create_uac2(pjsip_module *tsx_user,
 	return status;
 
 
-    /* Lock transaction. */
-    pj_grp_lock_acquire(tsx->grp_lock);
+    /* Lock transaction.
+     * We don't need to lock the group lock if none was supplied, while the
+     * newly created group lock has not been exposed.
+     */
+    if (grp_lock)
+        pj_grp_lock_acquire(tsx->grp_lock);
 
     /* Role is UAC. */
     tsx->role = PJSIP_ROLE_UAC;
@@ -1375,7 +1382,8 @@ PJ_DEF(pj_status_t) pjsip_tsx_create_uac2(pjsip_module *tsx_user,
      */
     status = pjsip_get_request_dest(tdata, &dst_info);
     if (status != PJ_SUCCESS) {
-	pj_grp_lock_release(tsx->grp_lock);
+	if (grp_lock)
+	    pj_grp_lock_release(tsx->grp_lock);
 	tsx_shutdown(tsx);
 	return status;
     }
@@ -1387,14 +1395,16 @@ PJ_DEF(pj_status_t) pjsip_tsx_create_uac2(pjsip_module *tsx_user,
 	/* The assertion is removed by #1090:
 	pj_assert(!"Bug in branch_param generator (i.e. not unique)");
 	*/
-	pj_grp_lock_release(tsx->grp_lock);
+	if (grp_lock)
+	    pj_grp_lock_release(tsx->grp_lock);
 	tsx_shutdown(tsx);
 	return status;
     }
 
 
     /* Unlock transaction and return. */
-    pj_grp_lock_release(tsx->grp_lock);
+    if (grp_lock)
+        pj_grp_lock_release(tsx->grp_lock);
 
     pj_log_push_indent();
     PJ_LOG(5,(tsx->obj_name, "Transaction created for %s",
@@ -2166,6 +2176,7 @@ PJ_DEF(pj_status_t) pjsip_tsx_retransmit_no_state(pjsip_transaction *tsx,
     pj_grp_lock_acquire(tsx->grp_lock);
     if (tdata == NULL) {
 	tdata = tsx->last_tx;
+	pjsip_tx_data_add_ref(tdata);
     }
     status = tsx_send_msg(tsx, tdata);
     pj_grp_lock_release(tsx->grp_lock);
@@ -3104,7 +3115,7 @@ static pj_status_t tsx_on_state_completed_uas( pjsip_transaction *tsx,
 	    /* Timer I is T4 timer for unreliable transports, and
 	     * zero seconds for reliable transports.
 	     */
-	    if (!tsx->is_reliable) {
+	    if (tsx->is_reliable) {
 		timeout.sec = 0; 
 		timeout.msec = 0;
 	    } else {
