@@ -1,4 +1,4 @@
-/* $Id: pjsua_vid.c 4903 2014-08-25 09:46:06Z nanang $ */
+/* $Id: pjsua_vid.c 5049 2015-04-07 09:25:07Z riza $ */
 /* 
  * Copyright (C) 2011-2011 Teluu Inc. (http://www.teluu.com)
  *
@@ -187,6 +187,8 @@ PJ_DEF(void) pjsua_vid_preview_param_default(pjsua_vid_preview_param *p)
     p->rend_id = PJMEDIA_VID_DEFAULT_RENDER_DEV;
     p->show = PJ_TRUE;
     p->wnd_flags = 0;
+    pj_bzero(&p->format, sizeof(p->format));
+    pj_bzero(&p->wnd, sizeof(p->wnd));
 }
 
 
@@ -437,6 +439,7 @@ static pj_status_t create_vid_win(pjsua_vid_win_type type,
 				  pjmedia_vid_dev_index cap_id,
 				  pj_bool_t show,
                                   unsigned wnd_flags,
+                                  const pjmedia_vid_dev_hwnd *wnd,
 				  pjsua_vid_win_id *id)
 {
     pj_bool_t enable_native_preview;
@@ -605,6 +608,10 @@ static pj_status_t create_vid_win(pjsua_vid_win_type type,
 	vp_param.vidparam.window_hide = !show;
         vp_param.vidparam.flags |= PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW_FLAGS;
         vp_param.vidparam.window_flags = wnd_flags;
+        if (wnd) {
+            vp_param.vidparam.flags |= PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW;
+            vp_param.vidparam.window = *wnd;
+        }
 
 	status = pjmedia_vid_port_create(w->pool, &vp_param, &w->vp_rend);
 	if (status != PJ_SUCCESS)
@@ -845,6 +852,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 				    PJSUA_INVALID_ID,
 				    acc->cfg.vid_in_auto_show,
                                     acc->cfg.vid_wnd_flags,
+                                    NULL,
 				    &wid);
 	    if (status != PJ_SUCCESS) {
 		pj_log_pop_indent();
@@ -915,6 +923,7 @@ pj_status_t pjsua_vid_channel_update(pjsua_call_media *call_med,
 					//acc->cfg.vid_cap_dev,
 					PJSUA_HIDE_WINDOW,
                                         acc->cfg.vid_wnd_flags,
+                                        NULL,
 					&wid);
 		if (status != PJ_SUCCESS) {
 		pj_log_pop_indent();
@@ -1066,6 +1075,7 @@ PJ_DEF(pj_status_t) pjsua_vid_preview_start(pjmedia_vid_dev_index id,
     pjsua_vid_win *w;
     pjmedia_vid_dev_index rend_id;
     pjsua_vid_preview_param default_param;
+    const pjmedia_format *fmt = NULL;
     pj_status_t status;
 
     if (!prm) {
@@ -1081,8 +1091,11 @@ PJ_DEF(pj_status_t) pjsua_vid_preview_start(pjmedia_vid_dev_index id,
 
     rend_id = prm->rend_id;
 
-    status = create_vid_win(PJSUA_WND_TYPE_PREVIEW, NULL, rend_id, id,
-			    prm->show, prm->wnd_flags, &wid);
+    if (prm->format.detail_type == PJMEDIA_FORMAT_DETAIL_VIDEO)
+	fmt = &prm->format;
+    status = create_vid_win(PJSUA_WND_TYPE_PREVIEW, fmt, rend_id, id,
+			    prm->show, prm->wnd_flags,
+			    (prm->wnd.info.window? &prm->wnd: NULL), &wid);
     if (status != PJ_SUCCESS) {
 	PJSUA_UNLOCK();
 	pj_log_pop_indent();
@@ -1237,20 +1250,24 @@ PJ_DEF(pj_status_t) pjsua_vid_win_get_info( pjsua_vid_win_id wid,
 
     PJSUA_LOCK();
     w = &pjsua_var.win[wid];
-
+    
     wi->is_native = w->is_native;
 
     if (w->is_native) {
 	pjmedia_vid_dev_stream *cap_strm;
 	pjmedia_vid_dev_cap cap = PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW;
 
-	cap_strm = pjmedia_vid_port_get_stream(w->vp_cap);
-	if (!cap_strm) {
+	if (!w->vp_cap) {
 	    status = PJ_EINVAL;
 	} else {
-	    status = pjmedia_vid_dev_stream_get_cap(cap_strm, cap, &wi->hwnd);
+	    cap_strm = pjmedia_vid_port_get_stream(w->vp_cap);
+	    if (!cap_strm) {
+		status = PJ_EINVAL;
+	    } else {
+		status = pjmedia_vid_dev_stream_get_cap(cap_strm, cap, 
+							&wi->hwnd);
+	    }
 	}
-
 	PJSUA_UNLOCK();
 	return status;
     }
@@ -1391,6 +1408,40 @@ PJ_DEF(pj_status_t) pjsua_vid_win_set_size( pjsua_vid_win_id wid,
     return status;
 }
 
+/*
+ * Set output window.
+ */
+PJ_DEF(pj_status_t) pjsua_vid_win_set_win( pjsua_vid_win_id wid,
+                                           const pjmedia_vid_dev_hwnd *win)
+{
+    pjsua_vid_win *w;
+    pjmedia_vid_dev_stream *s;
+    pj_status_t status;
+
+    PJ_ASSERT_RETURN(wid >= 0 && wid < PJSUA_MAX_VID_WINS && win, PJ_EINVAL);
+
+    PJSUA_LOCK();
+    w = &pjsua_var.win[wid];
+    if (w->vp_rend == NULL) {
+	/* Native window */
+	PJSUA_UNLOCK();
+	return PJ_EINVAL;
+    }
+
+    s = pjmedia_vid_port_get_stream(w->vp_rend);
+    if (s == NULL) {
+	PJSUA_UNLOCK();
+	return PJ_EINVAL;
+    }
+
+    status = pjmedia_vid_dev_stream_set_cap(s, 
+                            PJMEDIA_VID_DEV_CAP_OUTPUT_WINDOW, win);
+
+    PJSUA_UNLOCK();
+
+    return status;
+}
+
 /*
  * Set video orientation.
  */
@@ -1916,6 +1967,7 @@ static pj_status_t call_change_cap_dev(pjsua_call *call,
 				cap_dev,
 				PJSUA_HIDE_WINDOW,
 				acc->cfg.vid_wnd_flags,
+				NULL,
                                 &new_wid);
 	if (status != PJ_SUCCESS)
 	    goto on_error;
